import os
import tempfile

from zope.interface import Interface
from zc.lockfile import LockFile, LockError
from Products.Five import BrowserView

from collective.pdfLeadImage.async import get_queue

LOCKFILE_NAME = os.path.join(tempfile.gettempdir(),
                             __name__ + '.process_pdfLeadImage_queue')


class IpdfLeadImageQueueView(Interface):

    def process_conversion_queue(self):
        """process the queue.
        This is what you call from cron or zope clock server to get
        periodic image preview generation.
        """


class pdfLeadImageQueueView(BrowserView):

    def process_conversion_queue(self):
        """process the queue.
        """
        try:
            lock = LockFile(LOCKFILE_NAME)
        except LockError:
            return "`process_conversion_queue` is locked by another process (%r)." % (
                LOCKFILE_NAME)

        try:
            return self._process_conversion_queue()
        finally:
            lock.close()

    def _process_conversion_queue(self):
        msg = u''
        num = 0
        queue = get_queue('collective.pdfLeadImage.conversion')
        num = queue.process()
        if num:
            msg += u'%d Jobs in queue. Processing queue...\n' % num
            for job in queue.finished[-1:]:
                msg += str(job) + u'\n'
                msg += u'Finished Job'
        if num == 0:
            msg += u'No Jobs in Queue. No Action Required, None Taken.'
        return msg



from utils import addLeadImage
from utils import ghostscript_transform
from async import get_queue, Job

try:
    from collective.contentleadimage.config import IMAGE_FIELD_NAME
    from collective.contentleadimage.config import IMAGE_CAPTION_FIELD_NAME
    from collective.contentleadimage.interfaces import ILeadImageable
    import collective.contentleadimage
    LEADIMAGE_EXISTS = True
except ImportException:
    LEADIMAGE_EXISTS = False
    
def createLeadImage(object, event):
    """
        Creates a lead image from the pdf
    """
    #debug
    print("Running the handler for pdf lead image")
    if not LEADIMAGE_EXISTS:
        print("Problem importing lead image product")
    if not object.portal_type == 'File':
        print("Object is not a file")
    if object.getFile().getContentType() != "application/pdf":
        print("Object is not a pdf, it is a:" + object.getFile().getContentType())
    
    if LEADIMAGE_EXISTS and object.portal_type == 'File':
        field = object.getField(IMAGE_FIELD_NAME)
        if field is not None:
                    value = field.get(object)
                    hasLeadImage = not not value
        else:
            hasLeadImage = False
        
        if not hasLeadImage:
            file = str(object.getFile())
            if file != "":
                jpeg = ghostscript_transform(file, 1)
                addLeadImage(object, jpeg)
            else:
                print ("file is nothing")
                
def createLeadImageAsync(object, event):
    """
        Creates a lead image from the pdf asyncronously
    """
    #debug
    print("Running the handler for pdf lead image")
    if not LEADIMAGE_EXISTS:
        print("Problem importing lead image product")
    if not object.portal_type == 'File':
        print("Object is not a file")
    if object.getFile().getContentType() != "application/pdf":
        print("Object is not a pdf, it is a: " + object.getFile().getContentType())
    
    if LEADIMAGE_EXISTS and object.portal_type == 'File':
        field = object.getField(IMAGE_FIELD_NAME)
        if field is not None:
                    value = field.get(object)
                    hasLeadImage = not not value
        else:
            hasLeadImage = False
        
        if not hasLeadImage:
            file = str(object.getFile())
            if file != "":
                #ADD TO QUEUE HERE
                # get the queue
                conversion_queue = get_queue('collective.pdfLeadImage.conversion')
                # create a converter job
                converter_job = Job(createLeadImage, object)
                # add it to the queue
                conversion_queue.pending.append(converter_job)
                print("Document Conversion Job Queued")
            else:
                print ("file is nothing")



import subprocess
from Acquisition import aq_inner
try:
    from collective.contentleadimage.config import IMAGE_FIELD_NAME
    from collective.contentleadimage.config import IMAGE_CAPTION_FIELD_NAME
    from collective.contentleadimage.interfaces import ILeadImageable
    import collective.contentleadimage
    LEADIMAGE_EXISTS = True
except ImportException:
    LEADIMAGE_EXISTS = False

class FieldNotInitializedException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)


def addLeadImage(item, imageData):
    #set the lead image if necessary and if lead image product is installed
    if LEADIMAGE_EXISTS and imageData != "":
        print("I have data, creating lead image")
        #add the image as leadImage
        if ILeadImageable.providedBy(item):
            field = aq_inner(item).getField(IMAGE_FIELD_NAME)
            if field is not None:
                field.set(item, imageData)
                item.reindexObject()
                if field.getFilename(item) is None:
                    print "Added Image but the stupid filename field is still None"
            else:
                raise FieldNotInitializedException("The leadImage Field is None.")
        else:
            print("Item type does not accept leadImage")
            
        return

def ghostscript_transform(pdf, page_num):
    """
    ghostscript_transform
    args:
    pdf = pdf data
    page_num = number of the page to print to jpg
    returns:
    jpg data
    """
    print("Extracting page 1")
    
    first_page = "-dFirstPage=%s" % (page_num)
    last_page = "-dLastPage=%s" % (page_num)
    gs_cmd = [
        "gs",
        "-q",
        "-sDEVICE=jpeg",
        "-dJPEGQ=99",
        "-dGraphicsAlphaBits=4",
        "-dTextAlphaBits=4",
        "-dDOINTERPOLATE",
        "-dSAFER",
        "-dBATCH",
        "-dNOPAUSE",
        first_page,
        last_page,
        "-r59x56",
        "-sOutputFile=%stdout",
        "-",
        ]
    jpeg = None
    gs_process = subprocess.Popen(gs_cmd,stdout=subprocess.PIPE,stdin=subprocess.PIPE,)
    gs_process.stdin.write(pdf)
    jpeg = gs_process.communicate()[0]
    gs_process.stdin.close()
    return_code = gs_process.returncode
    if return_code == 0:
        print("Ghostscript processed one page of a pdf file.")
    else:
        print("Ghostscript process did not exit cleanly! Error Code: %d" % (return_code))
        jpeg = None
    return jpeg